﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using HeuristicLab.Common;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman1 : FeynmanDescriptor{
 public override string Name { get { return "Feynman I.6.2a f = exp(-theta^2/2)/sqrt(2*pi)"; } }

 protected override string TargetVariable { get { return "f"; } }
    protected override string[] VariableNames { get { return new string[] { "theta", "f" }; } }
    protected override string[] AllowedInputVariables { get { return new string[] { "theta" }; } }

    public int Seed { get; private set; }

    public Feynman1() : this((int)System.DateTime.Now.Ticks) { }

    public Feynman1(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint)Seed);

      var data = new List<List<double>>();
      var theta = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 3).ToList();

      var f = new List<double>();

      data.Add(theta);
      data.Add(f);


      for (var i = 0; i < theta.Count; i++) {
        var res = Math.Exp(Math.Pow(-theta[i], 2) / 2) / Math.Sqrt(2 * Math.PI);
        f.Add(res);
      }


      return data;
    }
  }
}
