using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman100 : FeynmanDescriptor {
    public override string Name { get { return "Feynman III.21.20 -rho_c_0*q*A_vec/m"; } }

    protected override string TargetVariable { get { return "j"; } }
    protected override string[] VariableNames { get { return new[] {"rho_c_0", "q", "A_vec", "m", "j"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"rho_c_0", "q", "A_vec", "m"}; } }

    public int Seed { get; private set; }

    public Feynman100() : this((int) DateTime.Now.Ticks) { }

    public Feynman100(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data    = new List<List<double>>();
      var rho_c_0 = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var q       = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var A_vec   = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var m       = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var j = new List<double>();

      data.Add(rho_c_0);
      data.Add(q);
      data.Add(A_vec);
      data.Add(m);
      data.Add(j);

      for (var i = 0; i < rho_c_0.Count; i++) {
        var res = -rho_c_0[i] * q[i] * A_vec[i] / m[i];
        j.Add(res);
      }

      return data;
    }
  }
}