﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using HeuristicLab.Common;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman11 : FeynmanDescriptor{
 public override string Name { get { return "Feynman I.12.4 Ef = q1*r/(4*pi*epsilon*r^3)"; } }

 protected override string TargetVariable { get { return "Ef"; } }
    protected override string[] VariableNames { get { return new string[] { "q1", "epsilon", "r", "Ef"}; } }
    protected override string[] AllowedInputVariables { get { return new string[] {"q1", "epsilon", "r"}; } }

    public int Seed { get; private set; }

    public Feynman11() : this((int)System.DateTime.Now.Ticks) { }

    public Feynman11(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint)Seed);

      var data = new List<List<double>>();
      var q1 = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var epsilon = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var r = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();


      var Ef = new List<double>();

      data.Add(q1);
      data.Add(epsilon);
      data.Add(r);
      data.Add(Ef);

      for (var i = 0; i < q1.Count; i++) {
        var res = q1[i] * r[i] / (4 * Math.PI * epsilon[i] * Math.Pow(r[i], 3));
        Ef.Add(res);
      }


      return data;
    }
  }
}
