﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using HeuristicLab.Common;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman12 : FeynmanDescriptor{
 public override string Name { get { return "Feynman I.12.5 F = q2*Ef"; } }

 protected override string TargetVariable { get { return "F"; } }
    protected override string[] VariableNames { get { return new string[] { "q2", "Ef", "F"}; } }
    protected override string[] AllowedInputVariables { get { return new string[] {"q2", "Ef"}; } }

    public int Seed { get; private set; }

    public Feynman12() : this((int)System.DateTime.Now.Ticks) { }

    public Feynman12(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint)Seed);

      var data = new List<List<double>>();
      var q2 = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var Ef = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var F = new List<double>();

      data.Add(q2);
      data.Add(Ef);
      data.Add(F);

      for (var i = 0; i < q2.Count; i++) {
        var res = q2[i] * Ef[i];
        F.Add(res);
      }

      return data;
    }
  }
}
