﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using HeuristicLab.Common;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman13 : FeynmanDescriptor{
 public override string Name { get { return "Feynman I.12.11 F = q*(Ef+B*v*sin(theta))"; } }

 protected override string TargetVariable { get { return "F"; } }
    protected override string[] VariableNames { get { return new string[] { "q", "Ef", "B", "v", "theta", "F"}; } }
    protected override string[] AllowedInputVariables { get { return new string[] {"q", "Ef", "B", "v", "theta"}; } }

    public int Seed { get; private set; }

    public Feynman13() : this((int)System.DateTime.Now.Ticks) { }

    public Feynman13(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint)Seed);

      var data = new List<List<double>>();
      var q = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var Ef = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var B = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var v = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var theta = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var F = new List<double>();

      data.Add(q);
      data.Add(Ef);
      data.Add(B);
      data.Add(v);
      data.Add(theta);
      data.Add(F);

      for (var i = 0; i < q.Count; i++) {
        var res = q[i] * (Ef[i] + B[i] * v[i] * Math.Sin(theta[i]));
        F.Add(res);
      }

      return data;
    }
  }
}
