using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman87 : FeynmanDescriptor {
    public override string Name { get { return "Feynman III.4.33 (h/(2*pi))*omega/(exp((h/(2*pi))*omega/(kb*T))-1)"; } }

    protected override string TargetVariable { get { return "E_n"; } }
    protected override string[] VariableNames { get { return new[] {"h", "omega", "kb", "T", "E_n"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"h", "omega", "kb", "T"}; } }

    public int Seed { get; private set; }

    public Feynman87() : this((int) DateTime.Now.Ticks) { }

    public Feynman87(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data  = new List<List<double>>();
      var h     = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var omega = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var kb    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var T     = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var E_n = new List<double>();

      data.Add(h);
      data.Add(omega);
      data.Add(kb);
      data.Add(T);
      data.Add(E_n);

      for (var i = 0; i < h.Count; i++) {
        var res = h[i] / (2 * Math.PI) * omega[i] / (Math.Exp((h[i] / (2 * Math.PI)) * omega[i] / (kb[i] * T[i])) - 1);
        E_n.Add(res);
      }

      return data;
    }
  }
}