using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman88 : FeynmanDescriptor {
    public override string Name { get { return "Feynman III.7.38 2*mom*B/(h/(2*pi))"; } }

    protected override string TargetVariable { get { return "omega"; } }
    protected override string[] VariableNames { get { return new[] {"mom", "B", "h", "omega"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"mom", "B", "h"}; } }

    public int Seed { get; private set; }

    public Feynman88() : this((int) DateTime.Now.Ticks) { }

    public Feynman88(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data = new List<List<double>>();
      var mom  = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var B    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var h    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var omega = new List<double>();

      data.Add(mom);
      data.Add(B);
      data.Add(h);
      data.Add(omega);

      for (var i = 0; i < mom.Count; i++) {
        var res = 2 * mom[i] * B[i] / (h[i] / (2 * Math.PI));
        omega.Add(res);
      }

      return data;
    }
  }
}