﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using HeuristicLab.Common;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman9 : FeynmanDescriptor{
 public override string Name { get { return "Feynman I.13.4 K = 1/2*m*(v^2+u^2+w^2)"; } }

 protected override string TargetVariable { get { return "K"; } }
    protected override string[] VariableNames { get { return new string[] { "m", "v", "u", "w", "K"}; } }
    protected override string[] AllowedInputVariables { get { return new string[] {"m", "v", "u", "w"}; } }

    public int Seed { get; private set; }

    public Feynman9() : this((int)System.DateTime.Now.Ticks) { }

    public Feynman9(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint)Seed);

      var data = new List<List<double>>();
      var m = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var v = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var u = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var w = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var K = new List<double>();

      data.Add(m);
      data.Add(v);
      data.Add(u);
      data.Add(w);
      data.Add(K);

      for (var i = 0; i < m.Count; i++) {
        var res = 1.0 / 2 * m[i] * (Math.Pow(v[i], 2) + Math.Pow(u[i], 2) + Math.Pow(w[i], 2));
        K.Add(res);
      }

      return data;
    }
  }
}
