using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman91 : FeynmanDescriptor {
    public override string Name { get { return "Feynman III.10.19 mom*sqrt(Bx**2+By**2+Bz**2)"; } }

    protected override string TargetVariable { get { return "E_n"; } }
    protected override string[] VariableNames { get { return new[] {"mom", "Bx", "By", "Bz", "E_n"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"mom", "Bx", "By", "Bz"}; } }

    public int Seed { get; private set; }

    public Feynman91() : this((int) DateTime.Now.Ticks) { }

    public Feynman91(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data = new List<List<double>>();
      var mom  = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var Bx   = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var By   = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var Bz   = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var E_n = new List<double>();

      data.Add(mom);
      data.Add(Bx);
      data.Add(By);
      data.Add(Bz);
      data.Add(E_n);

      for (var i = 0; i < mom.Count; i++) {
        var res = mom[i] * Math.Sqrt(Math.Pow(Bx[i], 2) + Math.Pow(By[i], 2) + Math.Pow(Bz[i], 2));
        E_n.Add(res);
      }

      return data;
    }
  }
}