using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman93 : FeynmanDescriptor {
    public override string Name { get { return "Feynman III.13.18 2*E_n*d**2*k/(h/(2*pi))"; } }

    protected override string TargetVariable { get { return "v"; } }
    protected override string[] VariableNames { get { return new[] {"E_n", "d", "k", "h", "v"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"E_n", "d", "k", "h"}; } }

    public int Seed { get; private set; }

    public Feynman93() : this((int) DateTime.Now.Ticks) { }

    public Feynman93(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data = new List<List<double>>();
      var E_n  = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var d    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var k    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var h    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var v = new List<double>();

      data.Add(E_n);
      data.Add(d);
      data.Add(k);
      data.Add(h);
      data.Add(v);

      for (var i = 0; i < E_n.Count; i++) {
        var res = 2 * E_n[i] * Math.Pow(d[i], 2) * k[i] / (h[i] / (2 * Math.PI));
        v.Add(res);
      }

      return data;
    }
  }
}