using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman94 : FeynmanDescriptor {
    public override string Name { get { return "Feynman III.14.14 I_0*(exp(q*Volt/(kb*T))-1)"; } }

    protected override string TargetVariable { get { return "I"; } }
    protected override string[] VariableNames { get { return new[] {"I_0", "q", "Volt", "kb", "T", "I"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"I_0", "q", "Volt", "kb", "T"}; } }

    public int Seed { get; private set; }

    public Feynman94() : this((int) DateTime.Now.Ticks) { }

    public Feynman94(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data = new List<List<double>>();
      var I_0  = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var q    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 2).ToList();
      var Volt = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 2).ToList();
      var kb   = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 2).ToList();
      var T    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 2).ToList();

      var I = new List<double>();

      data.Add(I_0);
      data.Add(q);
      data.Add(Volt);
      data.Add(kb);
      data.Add(T);
      data.Add(I);

      for (var i = 0; i < I_0.Count; i++) {
        var res = I_0[i] * (Math.Exp(q[i] * Volt[i] / (kb[i] * T[i])) - 1);
        I.Add(res);
      }

      return data;
    }
  }
}