using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman95 : FeynmanDescriptor {
    public override string Name { get { return "Feynman III.15.12 2*U*(1-cos(k*d))"; } }

    protected override string TargetVariable { get { return "E_n"; } }
    protected override string[] VariableNames { get { return new[] {"U", "k", "d", "E_n"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"U", "k", "d"}; } }

    public int Seed { get; private set; }

    public Feynman95() : this((int) DateTime.Now.Ticks) { }

    public Feynman95(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data = new List<List<double>>();
      var U    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var k    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var d    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var E_n = new List<double>();

      data.Add(U);
      data.Add(k);
      data.Add(d);
      data.Add(E_n);

      for (var i = 0; i < U.Count; i++) {
        var res = 2 * U[i] * (1 - Math.Cos(k[i] * d[i]));
        E_n.Add(res);
      }

      return data;
    }
  }
}