using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman96 : FeynmanDescriptor {
    public override string Name { get { return "Feynman III.15.14 (h/(2*pi))**2/(2*E_n*d**2)"; } }

    protected override string TargetVariable { get { return "m"; } }
    protected override string[] VariableNames { get { return new[] {"h", "E_n", "d", "m"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"h", "E_n", "d"}; } }

    public int Seed { get; private set; }

    public Feynman96() : this((int) DateTime.Now.Ticks) { }

    public Feynman96(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data = new List<List<double>>();
      var h    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var E_n  = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var d    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var m = new List<double>();

      data.Add(h);
      data.Add(E_n);
      data.Add(d);
      data.Add(m);

      for (var i = 0; i < h.Count; i++) {
        var res = h[i] / Math.Pow((2 * Math.PI), 2) / (2 * E_n[i] * Math.Pow(d[i], 2));
        m.Add(res);
      }

      return data;
    }
  }
}