using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman97 : FeynmanDescriptor {
    public override string Name { get { return "Feynman III.15.27 2*pi*alpha/(n*d)"; } }

    protected override string TargetVariable { get { return "k"; } }
    protected override string[] VariableNames { get { return new[] {"alpha", "n", "d", "k"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"alpha", "n", "d"}; } }

    public int Seed { get; private set; }

    public Feynman97() : this((int) DateTime.Now.Ticks) { }

    public Feynman97(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data  = new List<List<double>>();
      var alpha = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var n     = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var d     = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var k = new List<double>();

      data.Add(alpha);
      data.Add(n);
      data.Add(d);
      data.Add(k);

      for (var i = 0; i < alpha.Count; i++) {
        var res = 2 * Math.PI * alpha[i] / (n[i] * d[i]);
        k.Add(res);
      }

      return data;
    }
  }
}