using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman98 : FeynmanDescriptor {
    public override string Name { get { return "Feynman III.17.37 beta*(1+alpha*cos(theta))"; } }

    protected override string TargetVariable { get { return "f"; } }
    protected override string[] VariableNames { get { return new[] {"beta", "alpha", "theta", "f"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"beta", "alpha", "theta"}; } }

    public int Seed { get; private set; }

    public Feynman98() : this((int) DateTime.Now.Ticks) { }

    public Feynman98(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data  = new List<List<double>>();
      var beta  = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var alpha = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var theta = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var f = new List<double>();

      data.Add(beta);
      data.Add(alpha);
      data.Add(theta);
      data.Add(f);

      for (var i = 0; i < beta.Count; i++) {
        var res = beta[i] * (1 + alpha[i] * Math.Cos(theta[i]));
        f.Add(res);
      }

      return data;
    }
  }
}