using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman99 : FeynmanDescriptor {
    public override string Name {
      get { return "Feynman III.19.51 -m*q**4/(2*(4*pi*epsilon)**2*(h/(2*pi))**2)*(1/n**2)"; }
    }

    protected override string TargetVariable { get { return "E_n"; } }
    protected override string[] VariableNames { get { return new[] {"m", "q", "h", "n", "epsilon", "E_n"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"m", "q", "h", "n", "epsilon"}; } }

    public int Seed { get; private set; }

    public Feynman99() : this((int) DateTime.Now.Ticks) { }

    public Feynman99(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data    = new List<List<double>>();
      var m       = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var q       = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var h       = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var n       = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var epsilon = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var E_n = new List<double>();

      data.Add(m);
      data.Add(q);
      data.Add(h);
      data.Add(n);
      data.Add(epsilon);
      data.Add(E_n);

      for (var i = 0; i < m.Count; i++) {
        var res = -m[i] * Math.Pow(q[i], 4) / (2 * Math.Pow((4 * Math.PI * epsilon[i]), 2) * Math.Pow((h[i] / (2 * Math.PI)), 2) * (1.0 / Math.Pow(n[i], 2)));
        E_n.Add(res);
      }

      return data;
    }
  }
}